@@ -13,9 +13,10 @@
 #    under the License.
 
 import collections
+from collections import abc
 import logging
 
-import six
+import ovs.db.idl
 
 from ovsdbapp import api
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -34,14 +35,17 @@ class BaseCommand(api.Command):
     def execute(self, check_error=False, log_errors=True, **kwargs):
         try:
             if self.READ_ONLY:
-                self.run_idl(None)
-                return self.result
+                with self.api.ovsdb_connection.lock:
+                    self.run_idl(None)
+                    return self.result
             with self.api.transaction(check_error, log_errors, **kwargs) as t:
                 t.add(self)
             return self.result
         except Exception:
             if log_errors:
-                LOG.exception("Error executing command")
+                ignoring = "" if check_error else ": IGNORING"
+                LOG.exception("Error executing command (%s)%s",
+                              type(self).__name__, ignoring)
             if check_error:
                 raise
 
@@ -64,6 +68,8 @@ class BaseCommand(api.Command):
             ", ".join("%s=%s" % (k, v) for k, v in command_info.items()
                       if k not in ['api', 'result']))
 
+    __repr__ = __str__
+
 
 class ReadOnlyCommand(BaseCommand):
     READ_ONLY = True
@@ -75,14 +81,23 @@ class AddCommand(BaseCommand):
     def post_commit(self, txn):
         # If get_insert_uuid fails, self.result was not a result of a
         # recent insert. Most likely we are post_commit after a lookup()
-        real_uuid = txn.get_insert_uuid(self.result) or self.result
-        row = self.api.tables[self.table_name].rows[real_uuid]
+        if isinstance(self.result, rowview.RowView):
+            return
+        if isinstance(self.result, ovs.db.idl.Row):
+            row = self.result
+        else:
+            real_uuid = txn.get_insert_uuid(self.result) or self.result
+            # If we have multiple commands in a transation, post_commit can
+            # be called even if *this* command caused no change. Theoretically
+            # the subclass should have set a UUID/RowView result in that case
+            # which is handled above, so raise exception if real_uuid not found
+            row = self.api.tables[self.table_name].rows[real_uuid]
         self.result = rowview.RowView(row)
 
 
 class DbCreateCommand(BaseCommand):
     def __init__(self, api, table, _as_row=False, **columns):
-        super(DbCreateCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.columns = columns
         self.row = _as_row
@@ -104,7 +119,7 @@ class DbCreateCommand(BaseCommand):
 
 class DbDestroyCommand(BaseCommand):
     def __init__(self, api, table, record):
-        super(DbDestroyCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
 
@@ -115,7 +130,7 @@ class DbDestroyCommand(BaseCommand):
 
 class DbSetCommand(BaseCommand):
     def __init__(self, api, table, record, *col_values):
-        super(DbSetCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
         self.col_values = col_values
@@ -123,7 +138,7 @@ class DbSetCommand(BaseCommand):
     def run_idl(self, txn):
         record = self.api.lookup(self.table, self.record)
         for col, val in self.col_values:
-            if isinstance(val, collections.Mapping):
+            if isinstance(val, abc.Mapping):
                 # TODO(twilson) This is to make a unit/functional test that
                 # used OrderedDict work. In Python 3.7, insertion order is
                 # guaranteed to not change, but I need to verify this is is
@@ -147,7 +162,7 @@ class DbSetCommand(BaseCommand):
 
 class DbAddCommand(BaseCommand):
     def __init__(self, api, table, record, column, *values):
-        super(DbAddCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
         self.column = column
@@ -156,13 +171,13 @@ class DbAddCommand(BaseCommand):
     def run_idl(self, txn):
         record = self.api.lookup(self.table, self.record)
         for value in self.values:
-            if isinstance(value, collections.Mapping):
+            if isinstance(value, abc.Mapping):
                 # We should be doing an add on a 'map' column. If the key is
                 # already set, do nothing, otherwise set the key to the value
                 # Since this operation depends on the previous value, verify()
                 # must be called.
                 field = getattr(record, self.column, {})
-                for k, v in six.iteritems(value):
+                for k, v in value.items():
                     if k in field:
                         continue
                     field[k] = v
@@ -181,7 +196,7 @@ class DbAddCommand(BaseCommand):
 
 class DbClearCommand(BaseCommand):
     def __init__(self, api, table, record, column):
-        super(DbClearCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
         self.column = column
@@ -195,7 +210,7 @@ class DbClearCommand(BaseCommand):
 
 class DbGetCommand(ReadOnlyCommand):
     def __init__(self, api, table, record, column):
-        super(DbGetCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
         self.column = column
@@ -215,7 +230,7 @@ class DbGetCommand(ReadOnlyCommand):
 
 class DbListCommand(ReadOnlyCommand):
     def __init__(self, api, table, records, columns, if_exists, row=False):
-        super(DbListCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.columns = columns
         self.if_exists = if_exists
@@ -242,16 +257,23 @@ class DbListCommand(ReadOnlyCommand):
             rows = table_schema.rows.values()
 
         def _match(row):
+            elem = getattr(row, idx)
+            return elem in self.records
+
+        def _match_remove(row):
             elem = getattr(row, idx)
             found = elem in self.records
             if found:
                 records_found.remove(elem)
             return found
 
+        def _match_true(row):
+            return True
+
         records_found = []
         if idx and self.records:
             if self.if_exists:
-                match = lambda row: getattr(row, idx) in self.records
+                match = _match
             else:
                 # If we're using the approach of removing the unwanted
                 # elements, we'll use a helper list to remove elements as we
@@ -259,9 +281,9 @@ class DbListCommand(ReadOnlyCommand):
                 # quickly if there's some record missing to raise a RowNotFound
                 # exception later.
                 records_found = list(self.records)
-                match = _match
+                match = _match_remove
         else:
-            match = lambda row: True
+            match = _match_true
 
         self.result = [
             rowview.RowView(row) if self.row else {
@@ -278,7 +300,7 @@ class DbListCommand(ReadOnlyCommand):
 
 class DbFindCommand(ReadOnlyCommand):
     def __init__(self, api, table, *conditions, **kwargs):
-        super(DbFindCommand, self).__init__(api)
+        super().__init__(api)
         self.table = self.api._tables[table]
         self.conditions = conditions
         self.row = kwargs.get('row', False)
@@ -286,19 +308,21 @@ class DbFindCommand(ReadOnlyCommand):
                         list(self.table.columns.keys()) + ['_uuid'])
 
     def run_idl(self, txn):
+        # reduce search space if we have any indexed column and '=' match
+        rows = (idlutils.index_condition_match(self.table, *self.conditions) or
+                self.table.rows.values())
         self.result = [
             rowview.RowView(r) if self.row else {
                 c: idlutils.get_column_value(r, c)
                 for c in self.columns
             }
-            for r in self.table.rows.values()
-            if idlutils.row_match(r, self.conditions)
+            for r in rows if idlutils.row_match(r, self.conditions)
         ]
 
 
 class BaseGetRowCommand(ReadOnlyCommand):
     def __init__(self, api, record):
-        super(BaseGetRowCommand, self).__init__(api)
+        super().__init__(api)
         self.record = record
 
     def run_idl(self, txn):
@@ -307,7 +331,7 @@ class BaseGetRowCommand(ReadOnlyCommand):
 
 class DbRemoveCommand(BaseCommand):
     def __init__(self, api, table, record, column, *values, **keyvalues):
-        super(DbRemoveCommand, self).__init__(api)
+        super().__init__(api)
         self.table = table
         self.record = record
         self.column = column
@@ -332,5 +356,4 @@ class DbRemoveCommand(BaseCommand):
         except idlutils.RowNotFound:
             if self.if_exists:
                 return
-            else:
-                raise
+            raise
