@@ -15,6 +15,7 @@
 import logging
 
 from ovsdbapp.backend import ovs_idl
+from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp.backend.ovs_idl import transaction
 from ovsdbapp import exceptions
 from ovsdbapp.schema.open_vswitch import api
@@ -33,7 +34,7 @@ class OvsVsctlTransaction(transaction.Transaction):
         txn.expected_ifaces = set()
 
     def post_commit(self, txn):
-        super(OvsVsctlTransaction, self).post_commit(txn)
+        super().post_commit(txn)
         # ovs-vsctl only logs these failures and does not return nonzero
         try:
             self.do_post_commit(txn)
@@ -55,8 +56,9 @@ class OvsVsctlTransaction(transaction.Transaction):
             self.api.idl.wait(self.ovsdb_connection.poller)
             self.ovsdb_connection.poller.block()
         else:
-            raise exceptions.TimeoutException(commands=self.commands,
-                                              timeout=self.timeout)
+            raise exceptions.TimeoutException(
+                commands=self.commands, timeout=self.timeout,
+                cause='vsctl transaction did not end')
 
     def post_commit_failed_interfaces(self, txn):
         failed = []
@@ -75,6 +77,7 @@ class OvsVsctlTransaction(transaction.Transaction):
 
 class OvsdbIdl(ovs_idl.Backend, api.API):
     schema = 'Open_vSwitch'
+    lookup_table = idlutils._LOOKUP_TABLE
 
     @property
     def connection(self):
@@ -145,3 +148,9 @@ class OvsdbIdl(ovs_idl.Backend, api.API):
 
     def list_ifaces(self, bridge):
         return cmd.ListIfacesCommand(self, bridge)
+
+    def iface_get_external_id(self, name, field):
+        return cmd.IfaceGetExternalIdCommand(self, name, field)
+
+    def iface_set_external_id(self, name, field, value):
+        return cmd.IfaceSetExternalIdCommand(self, name, field, value)
