@@ -109,7 +109,8 @@ class API(api.API, metaclass=abc.ABCMeta):
         :param match:     The match rule
         :type match:      string
         :param action:    The action to take upon match
-        :type action:     'allow', 'allow-related', 'drop', or 'reject'
+        :type action:     'allow', 'allow-related', 'allow-stateless', 'drop',
+                          or 'reject'
         :param log:       If True, enable packet logging for the ACL
         :type log:        boolean
         :returns:         :class:`Command` with RowView result
@@ -146,22 +147,33 @@ class API(api.API, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def pg_acl_add(self, port_group, direction, priority, match, action,
-                   log=False):
+                   log=False, may_exist=False, severity=None, name=None,
+                   meter=None, **external_ids):
         """Add an ACL to 'port_group'
 
-        :param port_group: The name or uuid of the port group
-        :type port_group:  string or uuid.UUID
-        :param direction:  The traffic direction to match
-        :type direction:   'from-lport' or 'to-lport'
-        :param priority:   The priority field of the ACL
-        :type priority:    int
-        :param match:      The match rule
-        :type match:       string
-        :param action:     The action to take upon match
-        :type action:      'allow', 'allow-related', 'drop', or 'reject'
-        :param log:        If True, enable packet logging for the ACL
-        :type log:         boolean
-        :returns:          :class:`Command` with RowView result
+        :param port_group:   The name or uuid of the port group
+        :type port_group:    string or uuid.UUID
+        :param direction:    The traffic direction to match
+        :type direction:     'from-lport' or 'to-lport'
+        :param priority:     The priority field of the ACL
+        :type priority:      int
+        :param match:        The match rule
+        :type match:         string
+        :param action:       The action to take upon match
+        :type action:        'allow', 'allow-related', 'allow-stateless',
+                             'drop', or 'reject'
+        :param log:          If True, enable packet logging for the ACL
+        :type log:           boolean
+        :param may_exist:    If True, don't fail if the ACL already exists
+        :type may_exist:     boolean
+        :param severity:     Logging at alert, debug, info, notice or warning
+        :type severity:      string
+        :param name:         The name of the ACL
+        :type name:          string
+        :param meter:        The meter name to rate limit logging
+        :type meter:         string
+        :param external_ids: Values to be added as external_id pairs
+        :returns:            :class:`Command` with RowView result
         """
 
     @abc.abstractmethod
@@ -194,6 +206,68 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:          :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def address_set_add(self, name, addresses=None, may_exist=False):
+        """Create a set of addresses named 'name'
+
+        :param name:      The name of the address set
+        :type name:       string
+        :param addresses: One or more IP addresses to add to the address set
+        :type addresses:  list of strings
+        :param may_exist: If True, don't fail if the address set already exists
+        :type may_exist:  boolean
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_del(self, address_set, if_exists=False):
+        """Delete a set of addresses 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param if_exists:   Do not fail if the Address_set row does not exist
+        :type if_exists:    bool
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_get(self, address_set):
+        """Get a set of addresses for 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def address_set_list(self):
+        """Get all sets of addresses
+
+        :returns: :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def address_set_add_addresses(self, address_set, addresses):
+        """Add a list of addresses to 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to add to the address set
+        :type addresses:    string or list of strings
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def address_set_remove_addresses(self, address_set, addresses):
+        """Remove a list of addresses from 'address_set'
+
+        :param address_set: The name of the address set
+        :type address_set:  string or uuid.UUID
+        :param addresses:   One or more IP addresses to remove from address set
+        :type addresses:    string of list of strings
+        :returns:           :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def qos_add(self, switch, direction, priority, match, rate=None,
                 burst=None, dscp=None, may_exist=False, **columns):
@@ -535,6 +609,15 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:      :class:`Command` with RowView list result
         """
 
+    @abc.abstractmethod
+    def lrp_get(self, port):
+        """Get logical router port for 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :returns: :class:`Command` with RowView result
+        """
+
     @abc.abstractmethod
     def lrp_set_enabled(self, port, is_enabled):
         """Set administrative state of 'port'
@@ -575,6 +658,43 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:    :class:`Command` with dict result
         """
 
+    @abc.abstractmethod
+    def lrp_set_gateway_chassis(self, port, gateway_chassis, priority=0):
+        """Set gateway chassis for 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param priority:        The priority of the gateway chassis
+        :type priority:         int
+        :returns:               :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_get_gateway_chassis(self, port):
+        """Get the names of all gateway chassis on 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :type port:  int
+        :returns:    :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def lrp_del_gateway_chassis(self, port, gateway_chassis, if_exists=False):
+        """Delete gateway chassis from 'port'
+
+        :param port:            The name or uuid of the port
+        :type port:             string or uuid.UUID
+        :param gateway_chassis: The name of the gateway chassis
+        :type gateway_chassis:  string
+        :param if_exists:       If True, don't fail if the gateway chassis
+                                doesn't exist
+        :type if_exists:        boolean
+        :returns:               :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
@@ -678,7 +798,55 @@ class API(api.API, metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def lb_add(self, vip, ips, protocol=const.PROTO_TCP, may_exist=False):
+    def lr_policy_add(self, router, priority, match, action, may_exist=False,
+                      **columns):
+        """Add a routing policy (PBR) to 'router'
+
+        :param router:    The name or uuid of the router
+        :type router:     string or uuid.UUID
+        :param priority:  The priority of the policy
+        :type priority:   int
+        :param match:     The match rule
+        :type match:      string
+        :param action:    The action to take upon match
+        :type action:     POLICY_ACTION_ALLOW, POLICY_ACTION_DROP,
+                          POLICY_ACTION_REROUTE
+        :param may_exist: If True, don't fail if the policy already exists
+                          and instead update the columns
+        :type may_exist:  boolean
+        :param columns:   Additional columns to directly set on the policy
+        :returns:         :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lr_policy_del(self, router, priority=None, match=None,
+                      if_exists=False):
+        """Remove routing policies (PBR) from router
+
+        :param router:     The name or uuid of the router
+        :type router:      string or uuid.UUID
+        :param priority:   The priority of the rule
+        :type priority:    int
+        :param match:      The match condition of the rule. This should only
+                           be specified if priority is specified.
+        :type match:       string
+        :param if_exists:  If True, don't fail if the port doesn't exist
+        :type if_exists:   boolean
+        :returns:          :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lr_policy_list(self, router):
+        """Get the list of routing policies on 'router'
+
+        :param router: The name or uuid of the router
+        :type router:  string or uuid.UUID
+        :returns:      :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def lb_add(self, lb, vip, ips, protocol=const.PROTO_TCP, may_exist=False,
+               **columns):
         """Create a load-balancer or add a VIP to an existing load balancer
 
         :param lb:        The name or uuid of the load-balancer
@@ -692,6 +860,8 @@ class API(api.API, metaclass=abc.ABCMeta):
         :param may_exist: If True, don't fail if a LB w/ `vip` exists, and
                           instead, replace the vips on the LB
         :type may_exist:  boolean
+        :param columns:   Additional columns to directly set on the load
+                          balancer
         :returns:        :class:`Command` with RowView result
         """
 
@@ -1052,3 +1222,58 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type if_exists:   boolean
         :returns:          :class:`Command` with no result
         """
+
+    @abc.abstractmethod
+    def meter_add(self, name, unit, rate=1, fair=False, burst_size=0,
+                  action=None, may_exist=False, **columns):
+        """Create a Meter
+
+        :param name:        The name of the meter
+        :type name:         string
+        :param unit:        The unit of the meter (e.g kbps, pktps)
+        :type unit:         string
+        :param rate:        The rate of the meter
+        :type rate:         int
+        :param fair:        Specify whether meter rate limits its references
+                            individually (True) or as a shared pool (False)
+        :type fair:         boolean
+        :param burst_size:  The maximum burst allowed for the band in kilobits
+                            or packets, depending on the unit used
+        :type burst_size:   int
+        :param action:      The action of the meter. 'None' can be used as an
+                            alternate for the 'drop' value.
+        :type action:       string
+        :param may_exist:   If True, don't fail if the meter already exists
+        :type may_exist:    boolean
+        :param columns:     Additional columns to directly set on the meter
+                            (e.g external_ids)
+        :type columns:      dictionary
+        :returns:           :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def meter_del(self, meter, if_exists=False):
+        """Delete a meter
+
+        :param meter:       The name or uuid of the meter
+        :type meter:        string or uuid.UUID
+        :param if_exists:   If True, don't fail if the meter doesn't exist
+        :type if_exists:    boolean
+        :returns:           :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def meter_list(self):
+        """Get all meters
+
+        :returns:      :class:`Command` with RowView list result
+        """
+
+    @abc.abstractmethod
+    def meter_get(self, meter):
+        """Get the meter
+
+        :param meter:  The name or uuid of the meter
+        :type meter:   string or uuid.UUID
+        :returns:      :class:`Command` with RowView result
+        """
