@@ -80,6 +80,24 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def pg_acl_list(self, port_group):
         return cmd.PgAclListCommand(self, port_group)
 
+    def address_set_add(self, name, addresses=None, may_exist=False):
+        return cmd.AddressSetAddCommand(self, name, addresses, may_exist)
+
+    def address_set_del(self, address_set, if_exists=False):
+        return cmd.AddressSetDelCommand(self, address_set, if_exists)
+
+    def address_set_get(self, address_set):
+        return cmd.AddressSetGetCommand(self, address_set)
+
+    def address_set_list(self):
+        return cmd.AddressSetListCommand(self)
+
+    def address_set_add_addresses(self, address_set, addresses):
+        return cmd.AddressSetAddAddressesCommand(self, address_set, addresses)
+
+    def address_set_remove_addresses(self, address_set, addresses):
+        return cmd.AddressSetRemoveAddressCommand(self, address_set, addresses)
+
     def qos_add(self, switch, direction, priority, match, rate=None,
                 burst=None, dscp=None, external_ids_match=None,
                 may_exist=False, **columns):
@@ -182,6 +200,9 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lrp_list(self, router):
         return cmd.LrpListCommand(self, router)
 
+    def lrp_get(self, port):
+        return cmd.LrpGetCommand(self, port)
+
     def lrp_set_enabled(self, port, is_enabled):
         return cmd.LrpSetEnabledCommand(self, port, is_enabled)
 
@@ -194,6 +215,17 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lrp_get_options(self, port):
         return cmd.LrpGetOptionsCommand(self, port)
 
+    def lrp_set_gateway_chassis(self, port, gateway_chassis, priority=0):
+        return cmd.LrpSetGatewayChassisCommand(self,
+                                               port, gateway_chassis, priority)
+
+    def lrp_get_gateway_chassis(self, port):
+        return cmd.LrpGetGatewayChassisCommand(self, port)
+
+    def lrp_del_gateway_chassis(self, port, gateway_chassis, if_exists=False):
+        return cmd.LrpDelGatewayChassisCommand(self, port,
+                                               gateway_chassis, if_exists)
+
     def lr_route_add(self, router, prefix, nexthop, port=None,
                      policy='dst-ip', may_exist=False):
         return cmd.LrRouteAddCommand(self, router, prefix, nexthop, port,
@@ -218,6 +250,18 @@ class OvnNbApiIdlImpl(ovs_idl.Backend, api.API):
     def lr_nat_list(self, router):
         return cmd.LrNatListCommand(self, router)
 
+    def lr_policy_add(self, router, priority, match, action,
+                      may_exist=False, **columns):
+        return cmd.LrPolicyAddCommand(
+            self, router, priority, match, action, may_exist, **columns)
+
+    def lr_policy_del(self, router, priority=None, match=None,
+                      if_exists=False):
+        return cmd.LrPolicyDelCommand(self, router, priority, match, if_exists)
+
+    def lr_policy_list(self, router):
+        return cmd.LrPolicyListCommand(self, router)
+
     def lb_add(self, lb, vip, ips, protocol=const.PROTO_TCP, may_exist=False,
                **columns):
         return cmd.LbAddCommand(self, lb, vip, ips, protocol, may_exist,
