@@ -12,10 +12,10 @@
 
 import netaddr
 import testscenarios
+import uuid
 
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as const
-from ovsdbapp.schema.ovn_northbound import impl_idl
 from ovsdbapp.tests.functional import base
 from ovsdbapp.tests.functional.schema.ovn_northbound import fixtures
 from ovsdbapp.tests import utils
@@ -27,7 +27,7 @@ class OvnNorthboundTest(base.FunctionalTestCase):
 
     def setUp(self):
         super(OvnNorthboundTest, self).setUp()
-        self.api = impl_idl.OvnNbApiIdlImpl(self.connection)
+        self.api = self.useFixture(fixtures.NbApiFixture(self.connection)).obj
 
 
 class TestLogicalSwitchOps(OvnNorthboundTest):
@@ -236,11 +236,121 @@ class TestAclOps(OvnNorthboundTest):
         self.assertIn(r2, acls)
 
 
+class TestAddressSetOps(OvnNorthboundTest):
+    def setUp(self):
+        super(TestAddressSetOps, self).setUp()
+        self.table = self.api.tables['Address_Set']
+
+    def _addr_set_add(self, name=None, *args, **kwargs):
+        if name is None:
+            name = utils.get_rand_name()
+        fix = self.useFixture(fixtures.AddressSetFixture(self.api, name,
+                                                         *args, **kwargs))
+        self.assertIn(fix.obj.uuid, self.table.rows)
+        return fix.obj
+
+    def _test_addr_set_get(self, col):
+        addr_set = self._addr_set_add()
+        val = getattr(addr_set, col)
+        found = self.api.address_set_get(val).execute(check_error=True)
+        self.assertEqual(addr_set, found)
+
+    def test_addr_set_get_uuid(self):
+        self._test_addr_set_get('uuid')
+
+    def test_addr_set_get_name(self):
+        self._test_addr_set_get('name')
+
+    def test_addr_set_add_name(self):
+        name = utils.get_rand_device_name()
+        addr_set = self._addr_set_add(name)
+        self.assertEqual(name, addr_set.name)
+
+    def test_addr_set_add_exists(self):
+        name = utils.get_rand_device_name()
+        self._addr_set_add(name)
+        cmd = self.api.address_set_add(name)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def test_addr_set_add_may_exist(self):
+        name = utils.get_rand_device_name()
+        addr_set = self._addr_set_add(name)
+        addr_set2 = self.api.address_set_add(
+            name, may_exist=True).execute(check_error=True)
+        self.assertEqual(addr_set, addr_set2)
+
+    def test_addr_set_add_with_addresses(self):
+        addresses = ['192.168.0.1', '192.168.0.2']
+        addr_set = self._addr_set_add(addresses=addresses)
+        self.assertEqual(addresses, addr_set.addresses)
+
+    def test_addr_set_del(self):
+        addr_set = self._addr_set_add()
+        self.api.address_set_del(addr_set.uuid).execute(check_error=True)
+        self.assertNotIn(addr_set.uuid, self.table.rows)
+
+    def test_addr_set_del_by_name(self):
+        name = utils.get_rand_device_name()
+        self._addr_set_add(name)
+        self.api.address_set_del(name).execute(check_error=True)
+
+    def test_addr_set_del_no_exist(self):
+        name = utils.get_rand_device_name()
+        cmd = self.api.address_set_del(name)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def test_addr_set_del_if_exists(self):
+        name = utils.get_rand_device_name()
+        self.api.address_set_del(
+            name, if_exists=True).execute(check_error=True)
+
+    def test_addr_set_list(self):
+        addr_sets = {self._addr_set_add() for _ in range(3)}
+        found_sets = set(self.api.address_set_list().execute(check_error=True))
+        self.assertTrue(addr_sets.issubset(found_sets))
+
+    def test_addr_set_add_addresses(self):
+        addresses = ['192.168.0.1', '192.168.0.2']
+        addr_set = self._addr_set_add()
+
+        self.api.address_set_add_addresses(
+            addr_set.uuid, addresses).execute(check_error=True)
+        self.assertEqual(addresses, addr_set.addresses)
+
+        self.api.address_set_add_addresses(
+            addr_set.uuid, addresses).execute(check_error=True)
+        self.assertEqual(addresses, addr_set.addresses)
+
+    def test_addr_set_remove_addresses(self):
+        addresses = ['192.168.0.1', '192.168.0.2']
+        addr_set = self._addr_set_add(addresses=addresses)
+
+        self.api.address_set_remove_addresses(
+            addr_set.uuid, addresses).execute(check_error=True)
+        self.assertEqual(addr_set.addresses, [])
+
+        self.api.address_set_remove_addresses(
+            addr_set.uuid, addresses).execute(check_error=True)
+        self.assertEqual(addr_set.addresses, [])
+
+    def test_addr_set_add_remove_addresses_by_str(self):
+        address = "192.168.0.1"
+        addr_set = self._addr_set_add()
+
+        self.api.address_set_add_addresses(
+            addr_set.uuid, address).execute(check_error=True)
+        self.assertEqual([address], addr_set.addresses)
+
+        self.api.address_set_remove_addresses(
+            addr_set.uuid, address).execute(check_error=True)
+        self.assertEqual([], addr_set.addresses)
+
+
 class TestQoSOps(OvnNorthboundTest):
     def setUp(self):
         super(TestQoSOps, self).setUp()
         self.switch = self.useFixture(
-            fixtures.LogicalSwitchFixture(self.api)).obj
+            fixtures.LogicalSwitchFixture(self.api, name='LS_for_QoS')).obj
 
     def _qos_add(self, *args, **kwargs):
         cmd = self.api.qos_add(self.switch.uuid, *args, **kwargs)
@@ -252,7 +362,7 @@ class TestQoSOps(OvnNorthboundTest):
         self.assertEqual(cmd.rate, row.bandwidth.get('rate', None))
         self.assertEqual(cmd.burst, row.bandwidth.get('burst', None))
         self.assertEqual(cmd.dscp, row.action.get('dscp', None))
-        return row
+        return idlutils.frozen_row(row)
 
     def test_qos_add_dscp(self):
         self._qos_add('from-lport', 0, 'output == "fake_port" && ip', dscp=33)
@@ -287,10 +397,81 @@ class TestQoSOps(OvnNorthboundTest):
 
     def test_qos_add_may_exist(self):
         args = ('from-lport', 0, 'output == "fake_port" && ip', 1000)
-        row = self._qos_add(*args)
-        row2 = self._qos_add(*args, may_exist=True)
+        row = self._qos_add(*args, external_ids={'port_id': '1'})
+        row2 = self._qos_add(*args, external_ids={'port_id': '1'},
+                             may_exist=True)
         self.assertEqual(row, row2)
 
+    def test_qos_add_may_exist_update_using_external_ids_match(self):
+        args = ('from-lport', 0, 'output == "fake_port" && ip')
+        kwargs = {'rate': 1000, 'burst': 800, 'dscp': 16,
+                  'external_ids': {'port_id': '1'}}
+        row = self._qos_add(*args, **kwargs)
+
+        # Update QoS parameters: rate, burst and DSCP.
+        kwargs = {'rate': 1200, 'burst': 900, 'dscp': 24}
+        row2 = self._qos_add(*args, external_ids_match={'port_id': '1'},
+                             may_exist=True, **kwargs)
+        self.assertEqual(row.uuid, row2.uuid)
+        self.assertEqual(row2.bandwidth, {'rate': 1200, 'burst': 900})
+        self.assertEqual(row2.action, {'dscp': 24})
+
+        # Remove QoS parameters.
+        kwargs = {'rate': 1500, 'burst': 1100}
+        row3 = self._qos_add(*args, external_ids_match={'port_id': '1'},
+                             may_exist=True, **kwargs)
+        self.assertEqual(row.uuid, row3.uuid)
+        self.assertEqual(row3.bandwidth, {'rate': 1500, 'burst': 1100})
+        self.assertEqual(row3.action, {})
+
+        kwargs = {'rate': 2000}
+        row4 = self._qos_add(*args, external_ids_match={'port_id': '1'},
+                             may_exist=True, **kwargs)
+        self.assertEqual(row.uuid, row4.uuid)
+        self.assertEqual(row4.bandwidth, {'rate': 2000})
+        self.assertEqual(row4.action, {})
+
+        kwargs = {'dscp': 16}
+        row5 = self._qos_add(*args, external_ids_match={'port_id': '1'},
+                             may_exist=True, **kwargs)
+        self.assertEqual(row.uuid, row5.uuid)
+        self.assertEqual(row5.bandwidth, {})
+        self.assertEqual(row5.action, {'dscp': 16})
+
+    def test_qos_add_may_exist_using_external_ids_match(self):
+        _uuid = str(uuid.uuid4())
+        _uuid2 = str(uuid.uuid4())
+        for key in ('neutron:port_id', 'neutron:fip_id'):
+            args = ('from-lport', 0, 'output == "fake_port" && ip')
+            kwargs = {'rate': 1000, 'burst': 800, 'dscp': 16}
+            self._qos_add(*args, external_ids={key: _uuid}, **kwargs)
+
+            # "args" in this second call are different, "QoSAddCommand" will
+            # use the "external_ids_match" reference passed instead to match
+            # the QoS rules.
+            args = ('from-lport', 1, 'output == "fake_port" && ip')
+            self._qos_add(*args, external_ids_match={key: _uuid},
+                          may_exist=True, **kwargs)
+            qos_rules = self.api.qos_list(self.switch.uuid).execute(
+                check_error=True)
+            self.assertEqual(1, len(qos_rules))
+
+            # This call will update the "external_ids" to "_uuid2". Before
+            # changing it, "_uuid" will be used to find the register.
+            self._qos_add(*args, external_ids_match={key: _uuid},
+                          external_ids={key: _uuid2}, may_exist=True, **kwargs)
+            qos_rules = self.api.qos_list(self.switch.uuid).execute(
+                check_error=True)
+            self.assertEqual(1, len(qos_rules))
+
+            # The deletion call uses "_uuid2" because it was changed in the
+            # previous call.
+            self.api.qos_del_ext_ids(self.switch.name,
+                                     {key: _uuid2}).execute(check_error=True)
+            qos_rules = self.api.qos_list(self.switch.uuid).execute(
+                check_error=True)
+            self.assertEqual(0, len(qos_rules))
+
     def test_qos_add_extids(self):
         external_ids = {'mykey': 'myvalue', 'yourkey': 'yourvalue'}
         qos = self._qos_add('from-lport', 0, 'output == "fake_port" && ip',
@@ -308,8 +489,9 @@ class TestQoSOps(OvnNorthboundTest):
         r2 = self._qos_add('to-lport', 0, 'output == "fake_port"', 1000)
         self.api.qos_del(self.switch.uuid, 'from-lport').execute(
             check_error=True)
-        self.assertNotIn(r1, self.switch.qos_rules)
-        self.assertIn(r2, self.switch.qos_rules)
+        qos_rules = [idlutils.frozen_row(row) for row in self.switch.qos_rules]
+        self.assertNotIn(r1, qos_rules)
+        self.assertIn(r2, qos_rules)
 
     def test_qos_del_direction_priority_match(self):
         r1 = self._qos_add('from-lport', 0, 'output == "fake_port"', 1000)
@@ -317,8 +499,9 @@ class TestQoSOps(OvnNorthboundTest):
         cmd = self.api.qos_del(self.switch.uuid,
                                'from-lport', 0, 'output == "fake_port"')
         cmd.execute(check_error=True)
-        self.assertNotIn(r1, self.switch.qos_rules)
-        self.assertIn(r2, self.switch.qos_rules)
+        qos_rules = [idlutils.frozen_row(row) for row in self.switch.qos_rules]
+        self.assertNotIn(r1, qos_rules)
+        self.assertIn(r2, qos_rules)
 
     def test_qos_del_priority_without_match(self):
         self.assertRaises(TypeError, self.api.qos_del, self.switch.uuid,
@@ -340,6 +523,7 @@ class TestQoSOps(OvnNorthboundTest):
         r2 = self._qos_add('from-lport', 1, 'output == "fake_port2"', 1000)
         qos_rules = self.api.qos_list(self.switch.uuid).execute(
             check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertIn(r1, qos_rules)
         self.assertIn(r2, qos_rules)
 
@@ -354,18 +538,23 @@ class TestQoSOps(OvnNorthboundTest):
                                    dscp=10)
 
     def test_qos_delete_external_ids(self):
+        # NOTE(ralonsoh): the deletion method uses the LS name (first) and the
+        # UUID (second), in order to check that api.lookup() is working with
+        # both.
         self._create_fip_qoses()
         self.api.qos_del_ext_ids(self.switch.name,
                                  {'key1': 'value1'}).execute(check_error=True)
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_2, self.qos_3], qos_rules)
 
         self.api.qos_del_ext_ids(
-            self.switch.name,
+            self.switch.uuid,
             {'key3': 'value3', 'key4': 'value4'}).execute(check_error=True)
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_3], qos_rules)
 
     def test_qos_delete_external_ids_wrong_keys_or_values(self):
@@ -374,19 +563,22 @@ class TestQoSOps(OvnNorthboundTest):
                                  {'key_z': 'value1'}).execute(check_error=True)
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_1, self.qos_2, self.qos_3], qos_rules)
 
-        self.api.qos_del_ext_ids(self.switch.name,
+        self.api.qos_del_ext_ids(self.switch.uuid,
                                  {'key1': 'value_z'}).execute(check_error=True)
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_1, self.qos_2, self.qos_3], qos_rules)
 
         self.api.qos_del_ext_ids(
-            self.switch.name,
+            self.switch.uuid,
             {'key3': 'value3', 'key4': 'value_z'}).execute(check_error=True)
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_1, self.qos_2, self.qos_3], qos_rules)
 
     def test_qos_delete_external_ids_empty_dict(self):
@@ -404,6 +596,7 @@ class TestQoSOps(OvnNorthboundTest):
         # No qos rule has been deleted from the correct logical switch.
         qos_rules = self.api.qos_list(
             self.switch.uuid).execute(check_error=True)
+        qos_rules = [idlutils.frozen_row(row) for row in qos_rules]
         self.assertCountEqual([self.qos_1, self.qos_2, self.qos_3], qos_rules)
 
 
@@ -694,6 +887,9 @@ class TestDhcpOptionsOps(OvnNorthboundTest):
 
 
 class TestLogicalRouterOps(OvnNorthboundTest):
+    lr_policy_match1 = 'ip4.src==10.1.1.0/24 && ip4.dst==20.1.1.0/24'
+    lr_policy_match2 = 'ip4.src==30.1.1.0/24 && ip4.dst==40.1.1.0/24'
+
     def _lr_add(self, *args, **kwargs):
         lr = self.useFixture(
             fixtures.LogicalRouterFixture(self.api, *args, **kwargs)).obj
@@ -736,11 +932,13 @@ class TestLogicalRouterOps(OvnNorthboundTest):
                           may_exist=True)
         prefix = prefix or '192.0.2.0/25'
         nexthop = nexthop or '192.0.2.254'
+        port = port or "port_name"
         sr = self.api.lr_route_add(lr.uuid, prefix, nexthop, port,
                                    **kwargs).execute(check_error=True)
         self.assertIn(sr, lr.static_routes)
         self.assertEqual(prefix, sr.ip_prefix)
         self.assertEqual(nexthop, sr.nexthop)
+        self.assertIn(port, sr.output_port)
         sr.router = lr
         return sr
 
@@ -765,6 +963,11 @@ class TestLogicalRouterOps(OvnNorthboundTest):
         self._lr_add_route(router_name)
         self._lr_add_route(router_name, may_exist=True)
 
+    def test_lr_route_add_discard(self):
+        self._lr_add_route(nexthop=const.ROUTE_DISCARD)
+        self.assertRaises(netaddr.AddrFormatError, self._lr_add_route,
+                          prefix='not-discard')
+
     def test_lr_route_del(self):
         prefix = "192.0.2.0/25"
         route = self._lr_add_route(prefix=prefix)
@@ -965,6 +1168,153 @@ class TestLogicalRouterOps(OvnNorthboundTest):
         nats = self.api.lr_nat_list(lr.uuid).execute(check_error=True)
         self.assertEqual(lr.nat, nats)
 
+    def _lr_policy_add(self, priority, match, action, *args, **kwargs):
+        lr = kwargs.pop('router', self._lr_add(utils.get_rand_device_name()))
+        policy = self.api.lr_policy_add(
+            lr.uuid, priority, match, action, *args, **kwargs).execute(
+            check_error=True)
+        self.assertIn(policy, lr.policies)
+        self.assertEqual(action, policy.action)
+        self.assertEqual(match, policy.match)
+        self.assertEqual(priority, policy.priority)
+        policy.router = lr
+        return policy
+
+    def test_lr_policy_add_allow(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action)
+        self.assertEqual(policy.nexthop, [])
+        self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_add_drop(self):
+        priority = 10
+        action = const.POLICY_ACTION_DROP
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action)
+        self.assertEqual(policy.nexthop, [])
+        self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_add_reroute_nexthop(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        nexthop = "10.3.0.2"
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action,
+                                     nexthop=nexthop)
+        self.assertEqual(policy.nexthop, [nexthop])
+
+    def test_lr_policy_add_reroute_nexthops(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        nexthops = ["10.3.0.2", "10.3.0.3"]
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action,
+                                     nexthops=nexthops)
+        self.assertEqual(policy.nexthops, nexthops)
+
+    def test_lr_policy_add_invalid_action(self):
+        priority = 10
+        action = "allow-related"
+        self.assertRaises(TypeError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_invalid_priority(self):
+        priority = -1
+        action = "allow"
+        self.assertRaises(ValueError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_reroute_no_nexthop(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        self.assertRaises(ValueError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_nexthop_not_reroute(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        nexthop = "10.3.0.2"
+        self.assertRaises(ValueError, self._lr_policy_add, priority,
+                          self.lr_policy_match1, action, nexthop=nexthop)
+
+    def test_lr_policy_add_nexthops_not_reroute(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        nexthops = ["10.3.0.2", "10.3.0.3"]
+        self.assertRaises(ValueError, self._lr_policy_add, priority,
+                          self.lr_policy_match1, action, nexthops=nexthops)
+
+    def test_lr_policy_may_exist(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        policy_init = self._lr_policy_add(priority, self.lr_policy_match1,
+                                          action)
+        self.assertEqual(policy_init.nexthop, [])
+
+        # Update the policy
+        action = const.POLICY_ACTION_REROUTE
+        nexthop = "10.3.0.2"
+        policy_new = self._lr_policy_add(priority, self.lr_policy_match1,
+                                         action, nexthop=nexthop,
+                                         may_exist=True,
+                                         router=policy_init.router)
+        self.assertEqual(policy_init, policy_new)
+        self.assertEqual(policy_new.nexthop, [nexthop])
+
+    def _three_policies(self):
+        lr = self._lr_add(utils.get_rand_device_name())
+        pr1 = 10
+        pr2 = 9
+        self._lr_policy_add(pr1, self.lr_policy_match1,
+                            const.POLICY_ACTION_ALLOW, router=lr)
+        self._lr_policy_add(pr2, self.lr_policy_match2,
+                            const.POLICY_ACTION_DROP, router=lr)
+        self._lr_policy_add(pr1, self.lr_policy_match2,
+                            const.POLICY_ACTION_REROUTE,
+                            nexthops=['31.1.1.2', '31.1.1.3'], router=lr)
+        return lr
+
+    def _lr_policy_del(self, *args, **kwargs):
+        lr = self._three_policies()
+        self.api.lr_policy_del(lr.name,
+                               *args,
+                               **kwargs).execute(check_error=True)
+        return lr
+
+    def test_lr_policy_del_all(self):
+        lr = self._lr_policy_del()
+        self.assertEqual(lr.policies, [])
+
+    def test_lr_policy_del_priority(self):
+        priority = 10
+        lr = self._lr_policy_del(priority=priority)
+        self.assertEqual(len(lr.policies), 1)
+        self.assertEqual(lr.policies[0].action, const.POLICY_ACTION_DROP)
+
+    def test_lr_policy_del_match_without_priority(self):
+        self.assertRaises(TypeError, self._lr_policy_del,
+                          match='ip4.dst==40.1.1.0/24')
+
+    def test_lr_policy_del_specific(self):
+        priority = 10
+        lr = self._lr_policy_del(match=self.lr_policy_match2,
+                                 priority=priority)
+        for policy in lr.policies:
+            self.assertNotEqual(policy.action, const.POLICY_ACTION_REROUTE)
+            self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_del_not_found(self):
+        self.assertRaises(RuntimeError, self._lr_policy_del,
+                          priority=100, match='ip4.dst==40.1.1.0/24')
+
+    def test_lr_policy_del_if_exists(self):
+        lr = self._lr_policy_del(priority=100, match='ip4.dst==40.1.1.0/24',
+                                 if_exists=True)
+        self.assertEqual(len(lr.policies), 3)
+
+    def test_lr_policy_list(self):
+        lr = self._three_policies()
+        policies = self.api.lr_policy_list(lr.uuid).execute(check_error=True)
+        self.assertEqual(lr.policies, policies)
+
 
 class TestLogicalRouterPortOps(OvnNorthboundTest):
     def setUp(self):
@@ -984,6 +1334,18 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertEqual(set(networks), set(lrp.networks))
         return lrp
 
+    def _test_lrp_get(self, col):
+        lrp = self._lrp_add(None)
+        val = getattr(lrp, col)
+        found = self.api.lrp_get(val).execute(check_error=True)
+        self.assertEqual(lrp, found)
+
+    def test_lrp_get_uuid(self):
+        self._test_lrp_get('uuid')
+
+    def test_lrp_get_name(self):
+        self._test_lrp_get('name')
+
     def test_lrp_add(self):
         self._lrp_add(None, 'de:ad:be:ef:4d:ad', ['192.0.2.0/24'])
 
@@ -1066,6 +1428,15 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertIn(c1, lrp.gateway_chassis)
         self.assertIn(c2, lrp.gateway_chassis)
 
+    def test_gwc_add(self):
+        # NOTE: no API method to create gateway chassis directly
+        name, c1_name = [utils.get_rand_device_name() for _ in range(2)]
+        lrp = self._lrp_add(name, gateway_chassis=[c1_name])
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(c1.name, "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(c1.chassis_name, c1_name)
+        self.assertEqual(c1.priority, 1)
+
     def test_lrp_del_uuid(self):
         lrp = self._lrp_add(None)
         self.api.lrp_del(lrp.uuid).execute(check_error=True)
@@ -1131,6 +1502,55 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertEqual(options, self.api.lrp_get_options(lrp.uuid).execute(
             check_error=True))
 
+    def test_lrp_get_set_gateway_chassis(self):
+        lrp = self._lrp_add(None)
+        c1_name = utils.get_rand_device_name()
+        self.api.lrp_set_gateway_chassis(lrp.uuid, c1_name).execute(
+            check_error=True)
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+        lrp_gwcs = self.api.lrp_get_gateway_chassis(lrp.uuid).execute(
+            check_error=True)
+        self.assertIn(c1, lrp_gwcs)
+        self.assertEqual(c1.name, "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(c1.chassis_name, c1_name)
+        self.assertEqual(c1.priority, 0)
+
+    def test_lrp_set_multiple_gwc(self):
+        lrp = self._lrp_add(None)
+        c1_name, c2_name = [utils.get_rand_device_name() for _ in range(2)]
+        for gwc in [c1_name, c2_name]:
+            self.api.lrp_set_gateway_chassis(lrp.uuid, gwc).execute(
+                check_error=True)
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+        c2 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c2_name))
+        lrp_gwcs = self.api.lrp_get_gateway_chassis(lrp.uuid).execute(
+            check_error=True)
+        self.assertIn(c1, lrp_gwcs)
+        self.assertIn(c2, lrp_gwcs)
+
+    def test_lrp_del_gateway_chassis(self):
+        c1_name = utils.get_rand_device_name()
+        lrp = self._lrp_add(None, gateway_chassis=[c1_name])
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(lrp.gateway_chassis, [c1])
+        self.api.lrp_del_gateway_chassis(lrp.uuid, c1_name).execute(
+            check_error=True)
+        self.assertEqual(lrp.gateway_chassis, [])
+        self.assertRaises(idlutils.RowNotFound,
+                          self.api.lookup,
+                          'Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+
+    def test_lrp_del_gateway_chassis_wrong_chassis(self):
+        lrp = self._lrp_add(None)
+        cmd = self.api.lrp_del_gateway_chassis(lrp.uuid, "fake_chassis")
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def test_lrp_del_gateway_chassis_if_exist(self):
+        lrp = self._lrp_add(None)
+        self.api.lrp_del_gateway_chassis(
+            lrp.uuid, "fake_chassis", if_exists=True
+        ).execute(check_error=True)
+
 
 class TestLoadBalancerOps(OvnNorthboundTest):
 
